# Francois Massonnet
# January 25th 2015
#
# Some plots of averaged quantities from reproducibility experiments
#
# Input files are produced with the script of ECMean
graphics.off()
rm(list = ls())

library(ncdf)
source("/home/Earth/fmassonn/SCRIPTS/Rscripts/compute_extent.R")
source("/home/Earth/fmassonn/SCRIPTS/Rscripts/compute_cellarea.R")

exps  <- c('i077'  , 'm069', 'm06e', 'e011', 'e012', 'm06g')
labs  <- c('Ithaca', 'MN3', 'MN3-HiProc', 'ECMWF-HiProc', 'ECMWF-HiProc-options', 'MN3-HiProc-options')
#exps   <- c('i06c'  , 'm04y',         'e00y')
#labs   <- c('Ithaca', 'MareNostrum3', 'ECMWF')  

colors <- c(rgb(0.5, 0.0, 0), rgb(0.0, 0.5, 0.0), rgb(0.0, 1.0, 0.0), rgb(0.0, 0.0, 0.5), rgb(0.0, 0.0, 0.1), rgb(0.0, 0.75, 0.0))
memb   <- c('fc0', 'fc1', 'fc2', 'fc3', 'fc4')

n.memb <- length(memb)

yearb <- 1850
yeare <- 1869

rootdir <- '/esnas/scratch/fmassonn/REPRODUCIBILITY/'

# ===========
n.exp <- length(exps)
n.month <- 12
hemi <- c('N', 'S')
n.hemi <- 2
n.year <- yeare - yearb + 1
l_readgrid <- TRUE

sie <- array(NA, dim = c(n.exp, n.hemi, n.memb, n.year, n.month))
mst <- array(NA, dim = c(n.exp,         n.memb, n.year, n.month))
tp  <- array(NA, dim = c(n.exp,         n.memb, n.year, n.month))
for (j.exp in seq(1, n.exp)) {
  print(exps[j.exp])
  for (year in seq(yearb, yeare)) {
    print(year)
    for (j.memb in seq(1, n.memb)) {
      fcmemb <- memb[j.memb]
      filein <- paste(rootdir, exps[j.exp], '/18500101/', 
                fcmemb, '/Post_', toString(year), '/', 
                exps[j.exp], '_18500101_', fcmemb, '_',
                toString(year), '0101-', toString(year), '1231_ci.nc', sep = "")

      if (file.exists(filein)) {
        f <- open.ncdf(filein)
        sic <- get.var.ncdf(f, 'ci')
        if (l_readgrid) {
          lat <- get.var.ncdf(f, 'lat')
          lon <- get.var.ncdf(f, 'lon')
          tmp <- compute_cellarea(lon, lat)
          lon <- tmp$lon
          lat <- tmp$lat
          cellarea <- tmp$cellarea
          l_readgrid <- FALSE
        }
        close.ncdf(f)
        for (j.month in seq(1, n.month)) {
          sie[j.exp, 1, j.memb, year - yearb + 1, j.month] <- compute_extent(sic[,
              , j.month], lon, lat, cellarea, lsmask=NULL, sector="N")
          sie[j.exp, 2, j.memb, year - yearb + 1, j.month] <- compute_extent(sic[,
              , j.month], lon, lat, cellarea, lsmask=NULL, sector="S")
        }
      } else {
        sie[j.exp, 1, j.memb, year - yearb + 1, ] <- NA
        sie[j.exp, 2, j.memb, year - yearb + 1, ] <- NA
      }

      # Also open temperature
      filein <- paste(rootdir, exps[j.exp], '/18500101/',
                fcmemb, '/Post_', toString(year), '/',
                exps[j.exp], '_18500101_', fcmemb, '_',
                toString(year), '0101-', toString(year), '1231_tas.nc', sep = "")
      if (file.exists(filein)) {
        f <- open.ncdf(filein)     
        tas <- get.var.ncdf(f, 'tas') - 273.16
        for (j.month in seq(1, n.month)) {
          mst[j.exp,    j.memb, year - yearb + 1, j.month] <-
            sum(tas[, , j.month] * cellarea) / sum(cellarea)
        }
        close.ncdf(f)
      } else {
        mst[j.exp,    j.memb, year - yearb + 1, ] <- NA
      }


      # Also open precip
      filein <- paste(rootdir, exps[j.exp], '/18500101/',
                fcmemb, '/Post_', toString(year), '/',
                exps[j.exp], '_18500101_', fcmemb, '_',
                toString(year), '0101-', toString(year), '1231_totp.nc', sep = "")
      if (file.exists(filein)) {
        f <- open.ncdf(filein)     
        totp <- get.var.ncdf(f, 'totp') 
        close.ncdf(f)
        for (j.month in seq(1, n.month)) {
          tp[j.exp,     j.memb, year - yearb + 1, j.month] <-
              sum(totp[, , j.month] * cellarea )
        }
      } else {
        tp[j.exp,     j.memb, year - yearb + 1, ] <- NA
      }
    }
  }
}


# Plot sea ice mean & envelopes
me <- apply(sie, c(1, 2, 4, 5), mean)
ub <- apply(sie, c(1, 2, 4, 5), max)
lb <- apply(sie, c(1, 2, 4, 5), min)
for (j.hemi in seq(1, n.hemi)) {
  for (j.month in seq(1, n.month)) {
    setEPS()
    postscript(paste('../figs/sie_', hemi[j.hemi], '_m', sprintf("%02d", 
            j.month), '.eps', sep = ""), width = 5, height = 4)
    plot(NaN, NaN, xlim = c(yearb, yeare), 
         ylim = c(min(lb[, j.hemi, , j.month], na.rm = TRUE), 
                  max(ub[, j.hemi, , j.month], na.rm = TRUE)),
         xlab = 'Years', ylab = 'Million km²')
    for (j.exp in seq(1, n.exp)) {
      lines(seq(yearb, yeare), me[j.exp, j.hemi, , j.month], col = colors[j.exp], lwd = 5)
      lines(seq(yearb, yeare), ub[j.exp, j.hemi, , j.month], col = colors[j.exp], lwd = 1, lty = 2)
      lines(seq(yearb, yeare), lb[j.exp, j.hemi, , j.month], col = colors[j.exp], lwd = 1, lty = 2)
      text(yeare, max(ub[, j.hemi, , j.month]) - j.exp, labs[j.exp], col = colors[j.exp], pos = 2)
    }
    dev.off()
  }
} 

# Plot mst in annual mean envelopes
mst_anmean <- apply(mst, c(1, 2, 3), mean)
lb <- apply(mst_anmean, c(1, 3), min, na.rm = TRUE)
ub <- apply(mst_anmean, c(1, 3), max, na.rm = TRUE)
me <- apply(mst, c(1, 3), mean, na.rm = TRUE)

setEPS()
postscript(paste('../figs/tas.eps'), width = 5, height = 4)
plot(NaN, NaN, xlim = c(yearb, yeare), ylim = c(min(lb), max(ub)), 
    ylab = '°C', xlab = 'Years' )
for (j.exp in seq(1, n.exp)) {
  lines(seq(yearb, yeare), me[j.exp, ], col = colors[j.exp], lwd = 5)
  lines(seq(yearb, yeare), ub[j.exp, ], col = colors[j.exp], lwd = 1, lty = 2)
  lines(seq(yearb, yeare), lb[j.exp, ], col = colors[j.exp], lwd = 1, lty = 2)
}
dev.off()

# Plot totp in annual mean envelopes
mtp <- apply(tp, c(1, 2, 3), mean) * (365 * 86400)
me <- apply(mtp, c(1, 3), mean)
lb <- apply(mtp, c(1, 3), min)
ub <- apply(mtp, c(1, 3), max)

setEPS()
postscript(paste('../figs/precip.eps'), width = 5, height = 4)
plot(NaN, NaN, xlim = c(yearb, yeare), ylim = c(min(lb), max(ub)),
    ylab = 'kg', xlab = 'Years' )
for (j.exp in seq(1, n.exp)) {
  lines(seq(yearb, yeare), me[j.exp, ], col = colors[j.exp], lwd = 5)
  lines(seq(yearb, yeare), ub[j.exp, ], col = colors[j.exp], lwd = 1, lty = 2)
  lines(seq(yearb, yeare), lb[j.exp, ], col = colors[j.exp], lwd = 1, lty = 2)
}
dev.off()


# KS test
for (year in seq(yearb, yeare)) {
  toto <- ks.test(sie[1, 2, , year - yearb + 1, 9], sie[2, 2, , year - yearb + 1, 9])
  print('----------')
  print(year)
  print(sie[1, 2, , year - yearb + 1, 9])
  print(sie[2, 2, , year - yearb + 1, 9])
  print(toto$p.value)
}




